/**
 * Date:            Time:       User:               Description:
 * 16.03.2006       15:23       Smart Max           creation of the file IWritableNode.java
 *
 */
package org.max.trie;

import java.util.Set;
import java.util.List;

/**
 * IWritableNode
 *
 * @author  Maxim Podkolzine
 */
public interface IWritableNode extends INode {
    /**
     * @see     #addReference(IReference)
     */
    public List<IReference> getReferences();

    /**
     * Adds a new reference to the node
     *
     * @param   reference                           the new reference
     * @see     #getReferences()
     */
    public void addReference(IReference reference);

    /**
     * @see     #setDepth(int)
     */
    public int getDepth();

    /**
     * Sets the depth of the node
     *
     * @param   depth                               the new depth value
     * @see     #getDepth()
     */
    public void setDepth(int depth);

    /**
     * @see     #setParent(IWritableNode)
     */
    public IWritableNode getParent();

    /**
     * Sets the parent of the node
     *
     * @param   parent                              the new parent of node
     * @see     #getParent()
     */
    public void setParent(IWritableNode parent);

    /**
     * @see     #getOutcomingSymbols()
     * @see     #setIncomingSymbol(ISymbol)
     */
    public ISymbol getIncomingSymbol();

    /**
     * Sets the symbol on incoming edge of the node
     *
     * @param   incomingSymbol                      the new incoming symbol
     */
    public void setIncomingSymbol(ISymbol incomingSymbol);

    /**
     * @see     #hasLabel(ISymbol)
     * @see     #getChild(ISymbol)
     */
    public Set<ISymbol> getOutcomingSymbols();

    /**
     * @see     #getOutcomingSymbols()
     * @see     #getChild(ISymbol)
     */
    public boolean hasLabel(ISymbol label);

    /**
     * @see     #getOutcomingSymbols()
     * @see     #hasLabel(ISymbol)
     * @see     #addChild(IWritableNode)
     * @see     #changeChild(ISymbol, IWritableNode)
     */
    public IWritableNode getChild(ISymbol label);

    /**
     * Adds new child to the node.
     * <p>
     * Before invoking this method the <tt>node</tt> can have undefined parent (and it will not be updated),
     * but it must have appointed incoming symbol (see {@link #getIncomingSymbol()}). Exactly this symbol
     * will be written on the newly created edge. This symbol must be free for this node, otherwise an
     * exception will raise.
     * <p>
     * Right after this method, the child can be found using <tt>getChild(ISymbol)</tt> method.
     *
     * @param   node                                the new child of the node
     * @see     #getChild(ISymbol)
     * @see     #changeChild(ISymbol, IWritableNode)
     */
    public void addChild(IWritableNode node);

    /**
     * Changes the child of the node by symbol on edge.
     * <p>
     * Method updates the node which corresponds to specified <tt>symbol</tt>, the previous node
     * (and all its sub-trie) is lost from this trie, so it must be saved beforehand, if it is needed.
     * <p>
     * The <tt>symbol</tt> must exist among outcoming symbols of the node, or an exception will raise.
     *
     * @param   symbol                              the symbol to the child
     * @param   node                                new child
     * @see     #getChild(ISymbol)
     * @see     #addChild(IWritableNode)
     */
    public void changeChild(ISymbol symbol, IWritableNode node);
}
