/**
 * Date:            Time:       User:               Description:
 * 14.03.2006       21:02       Smart Max           creation of the file PatriciaTrieFactory.java
 *
 */
package org.max.trie.impl;

import org.max.trie.IPatriciaTrie;
import org.max.trie.IWritablePatriciaTrie;

/**
 * PatriciaTrieFactory
 *
 * @author  Maxim Podkolzine
 */
public class PatriciaTrieFactory {
    // Singleton attributes

    /**
     * Holds static instance of <tt>PatriciaTrieFactory</tt>
     */
    private static PatriciaTrieFactory ourInstance;

    /**
     * Does nothing. Constructor is private and the only possible way to create instance of this class
     * is to call method {@link #getInstance()}.
     */
    private PatriciaTrieFactory() {
    }

    /**
     * Returns the instance of <tt>PatriciaTrieFactory</tt>.
     * <p>
     * The first invocation entails the creation of static instance of this class, and
     * the further invocation will simply return it. Thus it is guarantee that this method
     * always returns the reference on same object.
     *
     * @return                                      the instance of <tt>PatriciaTrieFactory</tt>
     */
    public static PatriciaTrieFactory getInstance() {
        if (ourInstance == null) {
            ourInstance = new PatriciaTrieFactory();
        }

        return ourInstance;
    }




    // Main interface

    /**
     * Creates the instance of <tt>IPatriciaTrie</tt>
     *
     * @return                                      the instance of <tt>IPatriciaTrie</tt>
     */
    public IPatriciaTrie newPatriciaTrie() {
        return new PatriciaTrie();
    }

    /**
     * Creates the instance of <tt>IWritablePatriciaTrie</tt>
     *
     * @return                                      the instance of <tt>IWritablePatriciaTrie</tt>
     */
    public IWritablePatriciaTrie newWritablePatriciaTrie() {
        return new PatriciaTrie();
    }
}
