/**
 * Date:            Time:       User:               Description:
 * 04.05.2006       21:59       Smart Max           creation of the file IndexFabricDeserializer.java
 *
 */
package org.max.xml.index;

import static org.max.NativeSystem.*;

import net.sf.xanswer.indexing.IndexException;

import java.io.*;

import org.max.trie.ISymbol;
import org.max.trie.IPatriciaTrie;
import org.max.trie.impl.PatriciaTrieDeserializer;
import org.max.util.BidirectionalMap;

/**
 * IndexFabricDeserializer
 *
 * @author  Maxim Podkolzine
 */
public class IndexFabricDeserializer {
    /**
     * Reads the Index Fabric from specified location.
     * <p>
     * Method in turn reads attribute designators map, text designators map and tag designators map;
     * after this the Patricia trie is deserialized. The obtained maps and trie form the result Index Fabric.
     * <p>
     * Note, that the read Patricia trie is unmodifiable, thus the Index Fabric is too.
     *
     * @param   location                        the location to read from
     * @return                                  the deserialized Index Fabric
     * @throws  IndexException                  if errors while deserialization occur
     */
    public IndexFabric readIndexFabric(String location) throws IndexException {
        try {
            ObjectInput input = new ObjectInputStream(newInputStream(location + Separator + "symbols.dat"));
            BidirectionalMap<String, ISymbol> attrMap = (BidirectionalMap<String, ISymbol>) input.readObject();
            BidirectionalMap<String, ISymbol> textMap = (BidirectionalMap<String, ISymbol>) input.readObject();
            BidirectionalMap<String, ISymbol> tagMap = (BidirectionalMap<String, ISymbol>) input.readObject();
            input.close();

            PatriciaTrieDeserializer trieDeserializer = new PatriciaTrieDeserializer();
            IPatriciaTrie trie = trieDeserializer.readTrie(location);

            return new IndexFabric(trie, textMap, tagMap, attrMap);
        } catch (IOException e) {
            throw new IndexException(e);
        } catch (ClassNotFoundException e) {
            throw new IndexException(e);
        }
    }
}
