/**
 * Date:            Time:       User:               Description:
 * 07.04.2006       14:38:03       Smart Max           creation of the file XMLReference.java
 *
 */
package org.max.xml.index;

import org.max.trie.IReference;

import java.io.Serializable;

/**
 * XMLReference
 *
 * @author  Maxim Podkolzine
 */
public class XMLReference implements IReference, Serializable {
    private static final long serialVersionUID = 707139398002846369L;

    // Data

    /**
     * Holds the tag index (the number of tag in SAX traversing)
     */
    private int myTagIndex;

    /**
     * Holds the ID of document, from which the node appears
     */
    private String myDocID;




    // Constructor

    /**
     * Creates an instance of <tt>XMLReference</tt> with specified tag index and document ID
     *
     * @param   tagIndex                            the tag index
     * @param   docID                               the document ID
     */
    public XMLReference(int tagIndex, String docID) {
        myTagIndex = tagIndex;
        myDocID = docID;
    }




    // Main interface

    /**
     * Returns the tag index
     *
     * @return                                      the tag index
     */
    public int getTagIndex() {
        return myTagIndex;
    }

    /**
     * Returns the document ID
     *
     * @return                                      the document ID
     */
    public String getDocID() {
        return myDocID;
    }

    /**
     * Returns whether a specified object is equal to this XML reference.
     * <p>
     * Method returns <tt>true</tt> if and only if <tt>o</tt> is instance of <tt>XMLReference</tt>,
     * and has the same tag index and document ID.
     *
     * @param   o                                   the reference object with which to compare
     * @return                                      whether a specified object is equal to this one
     */
    public boolean equals(Object o) {
        if (!(o instanceof XMLReference)) {
            return false;
        }

        XMLReference that = (XMLReference) o;
        return (myTagIndex == that.myTagIndex &&
                myDocID != null ? myDocID.equals(that.myDocID) : that.myDocID == null);
    }
}
