/**
 * Date:            Time:       User:               Description:
 * 26.04.2006       21:11       Smart Max           creation of the file RandomXML.java
 *
 */
package org.max;

import static org.max.NativeSystem.*;

import java.util.LinkedList;
import java.util.Random;
import java.util.Set;
import java.util.HashSet;
import java.io.*;

/**
 * RandomXML
 */
public class RandomXML {
    private Random random = new Random();

    private String randomName(int length) {
        StringBuilder result = new StringBuilder();
        result.append((char)('a' + random.nextInt(26)));

        for (int i = 0; i < length - 1; i++) {
            if (random.nextBoolean()) {
                result.append((char)('a' + random.nextInt(26)));
            } else if (random.nextBoolean()) {
                result.append((char)('A' + random.nextInt(26)));
            } else if (random.nextBoolean()) {
                result.append((char)('0' + random.nextInt(10)));
            } else {
                result.append('_');
            }
        }

        return result.toString();
    }

    public void createXML(String fileName, int tagsNumber, int nameLength) throws IOException {
        assert(tagsNumber > 1);
        assert(nameLength > 0);

        BufferedWriter writer = new BufferedWriter(newWriter(fileName));
        writer.write("<root>");

        LinkedList<String> list = new LinkedList<String>();

        for (int i = 0; i < tagsNumber - 1; i++) {
            String name = randomName(nameLength);

            if (random.nextBoolean()) {
                list.addLast(name);
                list.addLast('/' + name);
            } else if (random.nextBoolean()) {
                list.addFirst('/' + name);
                list.addFirst(name);
            } else {
                list.addFirst(name);
                list.addLast('/' + name);
            }

            if (i % 50 == 0 || i == tagsNumber - 2) {
                for (String s : list) {
                    writer.write('<');
                    writer.write(s);
                    writer.write('>');
                }
                list.clear();
            }
        }

        writer.write("</root>");
        writer.flush();
        writer.close();
    }

    @SuppressWarnings({"unchecked"})
    public void createXML(String fileName, int tagsNumber, int differentTagsNumber, int nameLength,
                          int listsNumber, double attrProb, int attrLength,
                          double textProb, int textLength) throws IOException {
        assert(differentTagsNumber >= 2);
        assert(tagsNumber >= differentTagsNumber);
        assert(nameLength > 0);
        assert(listsNumber > 0);
        assert(listsNumber + 1 <= tagsNumber);
        assert(attrLength > 0);
        assert(textLength > 1);

        BufferedWriter writer = new BufferedWriter(newWriter(fileName));
        writer.write("<root>");

        Set<String> tagsNames = new HashSet<String>();
        while (tagsNames.size() < differentTagsNumber - 2) {
            String name = randomName(nameLength);

            if (!name.equals("root") && !name.equals("son")) {
                tagsNames.add(name);
            }
        }
        String[] tags = tagsNames.toArray(new String[differentTagsNumber - 2]);

        LinkedList<String>[] lists = new LinkedList[listsNumber];
        for (int i = 0; i < lists.length; i++) {
            lists[i] = new LinkedList<String>();
        }

        for (int i = 0; i < tagsNumber - listsNumber - 1; i++) {
            int index = random.nextInt(listsNumber);
            LinkedList<String> list = lists[index];

            int tagIndex = random.nextInt(differentTagsNumber - 2);
            String name = tags[tagIndex];

            String attr = null;
            if (random.nextDouble() < attrProb) {
                attr = randomName(attrLength) + " = \"" + randomName(textLength) + "\"";
            }

            String text = null;
            if (random.nextDouble() < textProb) {
                text = "." + randomName(textLength - 1);
            }

            if (random.nextBoolean()) {
                if (attr != null) {
                    list.addLast(name + " " + attr);
                } else {
                    list.addLast(name);
                }

                if (text != null) {
                    list.add(text);
                }

                list.addLast('/' + name);
            } else if (random.nextBoolean()) {
                list.addFirst('/' + name);

                if (text != null) {
                    list.add(text);
                }

                if (attr != null) {
                    list.addFirst(name + " " + attr);
                } else {
                    list.addFirst(name);
                }
            } else {
                if (attr != null) {
                    list.addFirst(name + " " + attr);
                } else {
                    list.addFirst(name);
                }

                if (text != null) {
                    list.add(text);
                }

                list.addLast('/' + name);
            }
        }

        for (LinkedList<String> list : lists) {
            writer.write("<son>");

            for (String s : list) {
                if (s.charAt(0) == '.') {
                    writer.write(s);
                } else {
                    writer.write('<');
                    writer.write(s);
                    writer.write('>');
                }
            }

            writer.write("</son>");
        }

        writer.write("</root>");
        writer.flush();
        writer.close();
    }
}
