/**
 * Date:            Time:       User:               Description:
 * 30.04.2006       16:51:30       Smart Max           creation of the file TestDivider1.java
 *
 */
package org.max.trie.impl;

import static org.max.NativeSystem.*;

import org.max.trie.INode;
import org.max.trie.ISymbol;
import org.max.trie.IPatriciaTrie;

import java.util.List;
import java.util.Collection;
import java.util.ArrayList;

/**
 * PatriciaTrieDivider
 */
class TestDivider1 {
    // Main interface

    private static int ourBlockSize = 5;

    public static int getBlockSize() {
        return TestDivider1.ourBlockSize;
    }

    public static void setBlockSize(int blockSize) {
        if (blockSize > 0) {
            TestDivider1.ourBlockSize = blockSize;
        }
    }

    // Divide method

    private String myDirectory;

    private int myBlock;

    private Block newBlock() {
        Block block = new Block((myBlock++) + ".block");
        myResultBlocks.add(block);
        return block;
    }

    private List<Block> myResultBlocks;

    /**
     * Returns the full size of sub-trie. No nodes of sub-trie are put in block.
     *
     * @param node
     * @return                                      the sub-trie size
     */
    private int getSubTrieSize(INode node) {
        int result = 1;

        for (ISymbol symbol : node.getOutcomingSymbols()) {
            INode child = node.getChild(symbol);
            result += getSubTrieSize(child);
        }

        return result;
    }

    /**
     * Puts the whole sub-trie to the specified block
     *
     * @param node
     * @param block
     */
    private void putSubTrie(INode node, Block block) {
        block.addNode(node);

        for (ISymbol symbol : node.getOutcomingSymbols()) {
            INode child = node.getChild(symbol);
            block.addEdge(node, child);
            putSubTrie(child, block);
        }

        assert(block.size() <= TestDivider1.ourBlockSize);
    }


    /**
     * Proceeds a sub-trie: divides it on blocks and returns the root block.
     *
     * @param rootNode
     * @return                                      the root block
     */
    private Block proceedSubTrie(Node rootNode) {
        int childrenNumber = rootNode.getOutcomingSymbols().size();
        int[] sizes = new int[childrenNumber];
        boolean[] used = new boolean[childrenNumber];
        INode[] nodes = new INode[childrenNumber];

        int i = 0;
        for (ISymbol symbol : rootNode.getOutcomingSymbols()) {
            nodes[i] = rootNode.getChild(symbol);
            sizes[i] = getSubTrieSize(nodes[i]);
            i++;
        }

        Block result = newBlock();
        result.addNode(rootNode);

        while (true) {
            int min = Integer.MAX_VALUE;
            int minJ = -1;
            for (int j = 0; j < sizes.length; j++) {
                if (!used[j] && sizes[j] < min) {
                    min = sizes[j];
                    minJ = j;
                }
            }

            if (minJ == -1) {
                break;
            }
            used[minJ] = true;
            Node child = (Node) nodes[minJ];

            if (min > TestDivider1.ourBlockSize) {                           // the sub-trie is too large, must be divided
                Block newBlock = proceedSubTrie(child);
                rootNode.changeChild(child.getIncomingSymbol(), myDirectory + newBlock.getName());
            } else {                                            // the sub-trie is appropriate to add as a whole
                if (result.size() + min <= TestDivider1.ourBlockSize) {
                    putSubTrie(child, result);                  // add all sub-trie of minJ-th child
                                                                // to the root block
                } else {
                    Block newBlock = newBlock();                // add all sub-trie of minJ-th child
                    putSubTrie(child, newBlock);                // to the separate block
                    rootNode.changeChild(child.getIncomingSymbol(), myDirectory + newBlock.getName());
                }
            }
        }

        // it must be guaranteed that size of resulting block <= ourBlockSize
        assert(result.size() <= TestDivider1.ourBlockSize);
        return result;
    }

    /**
     * Divides the trie on blocks.
     *
     * @param trie
     * @param directory
     * @return                                      the blocks, which divide the trie
     */
    public Collection<Block> divide(IPatriciaTrie trie, String directory) {
        myDirectory = directory + Separator;
        myBlock = 0;
        myResultBlocks = new ArrayList<Block>();

        proceedSubTrie((Node) trie.getRoot());

        return myResultBlocks;
    }
}
