/**
 * Date:            Time:       User:               Description:
 * 04.05.2006       21:22       Smart Max           creation of the file TestUtil.java
 *
 */
package org.max.trie.impl;

import org.max.trie.INode;
import org.max.trie.ISymbol;
import org.max.trie.IPatriciaTrie;
import org.max.trie.IWritablePatriciaTrie;

import junit.framework.TestCase;

import java.util.List;
import java.util.ArrayList;

/**
 * TestUtil
 */
abstract class TestUtil extends TestCase {
    public static void assertEqualSubTries(INode node1, INode node2) {
        assertEquals(node1.getDepth(), node2.getDepth());
        assertEquals(node1.getOutcomingSymbols().size(), node2.getOutcomingSymbols().size());
        assertEquals(node1.getReferences(), node2.getReferences());

        for (ISymbol symbol : node1.getOutcomingSymbols()) {
            INode child1 = node1.getChild(symbol);
            INode child2 = node2.getChild(symbol);

            assertEquals(child1.getIncomingSymbol(), child2.getIncomingSymbol());
            if (child1.getParent() != null) {
                assertEquals(child1.getParent(), node1);
            }
            if (child2.getParent() != null) {
                assertEquals(child2.getParent(), node2);
            }

            assertEqualSubTries(child1, child2);
        }
    }

    private static void addTrie(IWritablePatriciaTrie trie, INode node, List<ISymbol> key) {
        if (node.getReferences() != null) {
            trie.add(key);
        }

        for (ISymbol symbol : node.getOutcomingSymbols()) {
            INode child = node.getChild(symbol);

            key.add(symbol);
            addTrie(trie, child, key);
            key.remove(key.size() - 1);
        }
    }

    public static void addTrie(IWritablePatriciaTrie writableTrie, IPatriciaTrie trie) {
        addTrie(writableTrie, trie.getRoot(), new ArrayList<ISymbol>());
    }
}
