/**
 * Date:            Time:       User:               Description:
 * 17.03.2006       23:38       Smart Max           creation of the file PathBuilderTest.java
 *
 */
package org.max.xml.index;

import static org.w3c.dom.Node.*;

import junit.framework.TestCase;
import org.max.trie.ISymbol;
import java.util.List;

/**
 * PathBuilderTest
 */
public class PathBuilderTest extends TestCase {
    public void test_Tags() {
        List<ISymbol> list = PathBuilder.makeList("abc");

        assertEquals(3, list.size());
        assertEquals(new Symbol('a'), list.get(0));
        assertEquals(new Symbol('b'), list.get(1));
        assertEquals(new Symbol('c'), list.get(2));
    }

    public void test_Text() {
        List<ISymbol> list = PathBuilder.makeList(">abc");

        assertEquals(3, list.size());
        assertEquals(new Symbol('a', TEXT_NODE), list.get(0));
        assertEquals(new Symbol('b', TEXT_NODE), list.get(1));
        assertEquals(new Symbol('c', TEXT_NODE), list.get(2));
    }

    public void test_Attr() {
        List<ISymbol> list = PathBuilder.makeList("Z'p'>");

        assertEquals(2, list.size());
        assertEquals(new Symbol('Z', ATTRIBUTE_NODE), list.get(0));
        assertEquals(new Symbol('p', ATTRIBUTE_NODE), list.get(1));
    }

    public void test_TagAndText() {
        List<ISymbol> list = PathBuilder.makeList("ABC>Text");

        assertEquals(7, list.size());
        assertEquals(new Symbol('A'), list.get(0));
        assertEquals(new Symbol('B'), list.get(1));
        assertEquals(new Symbol('C'), list.get(2));
        assertEquals(new Symbol('T', TEXT_NODE), list.get(3));
        assertEquals(new Symbol('e', TEXT_NODE), list.get(4));
        assertEquals(new Symbol('x', TEXT_NODE), list.get(5));
        assertEquals(new Symbol('t', TEXT_NODE), list.get(6));
    }

    public void test_LongPath() {
        List<ISymbol> list = PathBuilder.makeList("XYZC'D'>Text");
        
        assertEquals(9, list.size());
        assertEquals(new Symbol('X'), list.get(0));
        assertEquals(new Symbol('Y'), list.get(1));
        assertEquals(new Symbol('Z'), list.get(2));
        assertEquals(new Symbol('C', ATTRIBUTE_NODE), list.get(3));
        assertEquals(new Symbol('D', ATTRIBUTE_NODE), list.get(4));
        assertEquals(new Symbol('T', TEXT_NODE), list.get(5));
        assertEquals(new Symbol('e', TEXT_NODE), list.get(6));
        assertEquals(new Symbol('x', TEXT_NODE), list.get(7));
        assertEquals(new Symbol('t', TEXT_NODE), list.get(8));
    }
}
