/**
 * Date:            Time:       User:               Description:
 * 18.03.2006       22:19       Smart Max           creation of the file XMLRetrieverTest.java
 *
 */
package org.max.xml.index;

import static org.max.NativeSystem.*;
import static org.max.xml.index.TestUtil.*;

import junit.framework.TestCase;

import java.util.Arrays;

import net.sf.xanswer.indexing.IndexException;

/**
 * XMLRetrieverTest
 */
public class XMLRetrieverTest extends TestCase {
    public void test_RetrieveFromDoc1() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        XMLBuilder.restoreDefaults();
        String docID = "xml" + Separator + "doc1.xml";
        xmlBuilder.addXMLDocument(docID);

        XMLRetriever retriever = new XMLRetriever(xmlBuilder.getIndexFabric());

        // retrieve tags
        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("invoice")));
        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("invoice", "buyer", "name")));
        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("invoice", "buyer", "address")));
        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("invoice", "seller", "name")));
        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("invoice", "seller", "address")));
        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("invoice", "item")));

        // retrieve attribute
        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("invoice", "item"), "count", ""));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("invoice", "item"), "count", "2"), docID);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("invoice", "item"), "count", "3"), docID);

        // retrieve text
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("invoice", "buyer", "name"), "ABC Corp"), docID);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("invoice", "buyer", "address"), "1 Industrial Way"),
                docID);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("invoice", "seller", "name"), "Acme Inc"), docID);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("invoice", "seller", "address"), "2 Acme Rd."), docID);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("invoice", "item"), "drill"), docID);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("invoice", "item"), "saw"), docID);
    }

    public void test_ManyXML() throws IndexException {
        String[] docIDs = new String[]{
                "xml" + Separator + "attr.xml",
                "xml" + Separator + "blanks.xml",
                "xml" + Separator + "similarAttrs.xml",
                "xml" + Separator + "similarTags.xml",
                "xml" + Separator + "tags.xml",
                "xml" + Separator + "text.xml",
                "xml" + Separator + "uppercase.xml"
        };

        XMLBuilder builder = new XMLBuilder();
        for (String docID : docIDs) {
            builder.addXMLDocument(docID);
        }

        XMLRetriever retriever = new XMLRetriever(builder.getIndexFabric());

        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("name", "name1", "name2")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("name", "name1"), "name1", "name"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("name", "name1", "name2"), "name", "name"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("name", "name1", "name2"), "y", "1"), docIDs[0]);

        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("thename", "first"), "b", "temp"), docIDs[1]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("thename", "second"), " "), docIDs[1]);

        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("document", "tag"), "b", ""), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("document", "tag"), "aaa", ""), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("document", "tag"), "bbb", ""), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("document", "v", "u", "w"), "aa", ""), docIDs[2]);

        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("ada", "a", "ad", "da")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("ada", "a", "ad", "da", "d", "aad")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("ada", "d", "aa", "daa", "daad")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("ada", "d", "aa", "daa", "adad")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("ada", "d", "aa", "daa", "a")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("ada", "dd", "aad", "d", "daad")));

        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("tag", "tag2", "t2")));
        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("tag", "tag2", "tt")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("tag", "tag2", "tt"), "attr", ""), docIDs[4]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("tag", "tag2", "tt", "t")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("tag", "tag2"), "Another text"), docIDs[4]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("tag", "tag0"), "attr", ""), docIDs[4]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("tag", "tt0")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("tag", "taag"), "Text"), docIDs[4]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("tag", "attr")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("tag", "agt", "tagg")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("tag", "Tag")));

        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("text", "sample1")));
        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("text", "sample2")));
        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("text", "sample3")));
        checkInternalNode(retriever.retrieveRawPath(Arrays.asList("text", "sample4")));
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("text"), "\n    "), docIDs[5]);

        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("name", "Name"), "1"), docIDs[6]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("name", "NAME"), "2"), docIDs[6]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("name", "nAmE"), "3"), docIDs[6]);
        checkLeafNode(retriever.retrieveRawPath(Arrays.asList("name", "namE"), "4"), docIDs[6]);
    }
}
